/* eslint-disable no-console */
import type { HighlighterCore, LanguageRegistration, ThemeInput, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { createContext, useContext, useEffect, useMemo, useState } from 'react'

const DEBUG = true
function _log(...args: any[]) {
  if (DEBUG)
    console.log('[ShikiProvider]', ...args)
}

interface ShikiContextValue {
  highlighter: HighlighterCore | null
  isReady: boolean
  error: Error | null
}

interface ShikiProviderProps {
  children: React.ReactNode
  initialLanguages: LanguageRegistration[]
  initialThemes: ThemeInput[]
}

const ShikiContext = createContext<ShikiContextValue | null>(null)

export function ShikiProvider({ children, initialLanguages, initialThemes }: ShikiProviderProps) {
  const [highlighter, setHighlighter] = useState<HighlighterCore | null>(null)
  const [isReady, setIsReady] = useState(false)
  const [error, setError] = useState<Error | null>(null)

  useEffect(() => {
    const initHighlighter = async () => {
      try {
        const highlighterInstance = await createHighlighterCore({
          engine: createJavaScriptRegexEngine(),
        })

        // Fallback for the first theme if specific theme access is required
        const activeTheme = initialThemes[0] as ThemeRegistrationAny

        // Test code tokenization with the dynamically set theme
        const testCode = `
          // This is a comment
          function test() {
            const str = "hello";
            return str;
          }
        `

        const testTokens = highlighterInstance.codeToTokens(testCode.trim(), {
          lang: 'typescript',
          theme: activeTheme,
          includeExplanation: true,
        })

        _log('Test tokenization:', {
          tokens: testTokens.tokens[0]?.map(token => ({
            content: token.content,
            color: token.color,
            scopes: token.explanation?.[0]?.scopes.map(s => s.scopeName),
            matches: token.explanation?.[0]?.scopes
              .flatMap(s => s.themeMatches || [])
              .map(m => ({
                scope: m.scope,
                color: m.settings.foreground,
              })),
          })),
          theme: highlighterInstance.getTheme(activeTheme),
        })

        setHighlighter(highlighterInstance)
        setIsReady(true)
        setError(null)
      }
      catch (err) {
        console.error('Highlighter initialization failed:', err)
        setError(err instanceof Error ? err : new Error('Initialization failed'))
        setIsReady(false)
      }
    }

    initHighlighter()
  }, [initialLanguages, initialThemes])

  const value = useMemo<ShikiContextValue>(
    () => ({
      highlighter,
      isReady,
      error,
    }),
    [highlighter, isReady, error],
  )

  return <ShikiContext.Provider value={value}>{children}</ShikiContext.Provider>
}

export function useShiki() {
  const context = useContext(ShikiContext)
  if (!context) {
    throw new Error('useShiki must be used within a ShikiProvider')
  }
  return context
}
